package com.yihu.hos.logger.camel.processor;

import ch.qos.logback.classic.spi.ILoggingEvent;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Map;

/**
 * ehr log处理
 *
 * @author Airhead
 * @since 2017/3/30.
 */
public class EHRLogProcesser implements Processor {
    @Override
    public void process(Exchange exchange) throws Exception {
        exchange.getOut().setBody("");
        exchange.getOut().setHeader("empty", true);

        ArrayList arrayList = exchange.getIn().getBody(ArrayList.class);
        if (arrayList.size() != 4) {
            return;
        }

        Object first = arrayList.get(0);
        Object last = arrayList.get(arrayList.size() - 1);

        try {
            ILoggingEvent request = (ILoggingEvent) first;
            ILoggingEvent response = (ILoggingEvent) last;

            Map<String, String> mdcPropertyMap = request.getMDCPropertyMap();
            if (mdcPropertyMap == null || !mdcPropertyMap.get("camel.routeId").equals("api")) {
                return;
            }

            ObjectMapper objectMapper = new ObjectMapper();
            JsonNode requestNode = objectMapper.readValue(request.getMessage(), JsonNode.class);
            JsonNode responseNode = objectMapper.readValue(response.getMessage(), JsonNode.class);


            JsonNode inNode = requestNode.with("in");
            JsonNode outNode = responseNode.with("out");

            ObjectNode objectNode = objectMapper.createObjectNode();
            objectNode.put("time", new Timestamp(request.getTimeStamp()).toString());
            objectNode.put("logType", 1);
            objectNode.put("caller", inNode.get("appKey").asText());

            ObjectNode dataNode = objectMapper.createObjectNode();
            dataNode.put("responseTime", response.getTimeStamp() - request.getTimeStamp());
            dataNode.put("url", inNode.get("CamelHttpUrl").asText());
            dataNode.put("responseCode", outNode.get("CamelHttpResponseCode").asText());
            dataNode.put("response", outNode.get("body").asText());
            dataNode.put("api", inNode.get("api").asText());
            dataNode.put("appKey", inNode.get("appKey").asText());

            ObjectNode paramNode = objectMapper.createObjectNode();
            paramNode.put("api", inNode.get("api").asText());
            paramNode.put("appKey", inNode.get("appKey").asText());
            paramNode.put("sign", inNode.get("sign").asText());
            paramNode.put("timestamp", inNode.get("timestamp").asText());
            paramNode.put("v", inNode.get("v").asText());
            if (inNode.has("param")) {
                paramNode.put("param", inNode.get("param").asText());
            }

            if (inNode.has("token")) {
                paramNode.put("token", inNode.get("token").asText());
            }

            dataNode.set("params", paramNode);
            objectNode.set("data", dataNode);

            String content = objectMapper.writeValueAsString(objectNode);

            exchange.getOut().setBody(content);
            exchange.getOut().setHeader("empty", false);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
