package com.yihu.hos.logger.camel.route;

import com.yihu.hos.logger.camel.aggregator.EHRLogAggregator;
import com.yihu.hos.logger.camel.expression.EHRLogExpression;
import com.yihu.hos.logger.camel.processor.EHRLogProcesser;
import com.yihu.hos.logger.configuration.LoggerConfiguration;
import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.component.jms.JmsComponent;
import org.apache.camel.model.ModelCamelContext;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jms.ConnectionFactory;

/**
 * @author Airhead
 * @since 2017/3/30.
 */
//@Component
public class EHRLogRouterBuilder extends RouteBuilder {
    private static final String DESTINATION_NAME = "business-log-ehr";
    private static final String QUEUE_NAME = "business.log.queue";
    @Autowired
    private LoggerConfiguration loggerConfiguration;

    @Override
    public void configure() throws Exception {
        ModelCamelContext context = this.getContext();

        ConnectionFactory connectionFactory = new ActiveMQConnectionFactory(
                loggerConfiguration.getActiveMqUser(), loggerConfiguration.getActiveMqPassword(), loggerConfiguration.getBrokerURL());
        context.addComponent(DESTINATION_NAME, JmsComponent.jmsComponentAutoAcknowledge(connectionFactory));
        from(DESTINATION_NAME + ":topic:" + QUEUE_NAME)
                .aggregate(method(new EHRLogExpression(), "getBreadcrumbId"), new EHRLogAggregator())
                .completionSize(4)
                .completionTimeout(20 * 1000)
                .process(new EHRLogProcesser())
                .choice()
                .when(header("empty").isNotEqualTo(true)).to("file://" + loggerConfiguration.getEhrLogDir())
                .endChoice();
    }
}
