package com.yihu.hos.logger.service;

import ch.qos.logback.classic.spi.ILoggingEvent;
import com.yihu.hos.logger.configuration.LoggerConfiguration;
import com.yihu.hos.logger.model.HosLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Map;

/**
 * @created Airhead 2016/8/8.
 */
@Component("businessLogService")
public class HosLogService {
    @Autowired
    private MongoOperations mongoOperations;

    @Autowired
    private LoggerConfiguration loggerConfiguration;

    /**
     * camel.exchangeId The exchange id
     * camel.messageId The message id
     * camel.correlationId The correlation id of the exchange if it's correlated. For example a sub message from the Splitter EIP
     * camel.transactionKey The id of the transaction for transacted exchanges.
     * Note the id is not unique, but its the id of the transaction template that marks the transaction boundary for the given transaction.
     * Hence we decided to name the key transactionKey and not transactionID to point out this fact.
     * camel.routeId The id of the route, in which the exchange is currently being routed
     * camel.breadcrumbId An unique id used for tracking messages across transports.
     * camel.contextId The camel context id used for tracking the message from different camel context.
     *
     * @param event
     */
    public void log(ILoggingEvent event) {
        Map<String, String> mdcPropertyMap = event.getMDCPropertyMap();
        HosLog hosLog = new HosLog();

        hosLog.setFireTime(new Date(event.getTimeStamp()));
        hosLog.setExchangeId(mdcPropertyMap.get("camel.exchangeId"));
        hosLog.setMessageId(mdcPropertyMap.get("camel.messageId"));
        hosLog.setCorrelationId(mdcPropertyMap.get("camel.correlationId"));
        hosLog.setTransactionKey(mdcPropertyMap.get("camel.transactionKey"));
        hosLog.setRouteId(mdcPropertyMap.get("camel.routeId"));
        hosLog.setBreadcrumbId(mdcPropertyMap.get("camel.breadcrumbId"));
        hosLog.setCamelContextId(mdcPropertyMap.get("camel.contextId"));
        hosLog.setBody(event.getMessage());
        hosLog.setBodyLength(event.getMessage().length());
        hosLog.setTenant(loggerConfiguration.getTenant());

        mongoOperations.save(hosLog);
    }
}
